Attribute VB_Name = "RTF_PrintingModule"
'****************************************************************
'*                  ACKNOWLEDGEMENTS                            *
'*                                                              *
'*   Code Guru Website                                          *
'*   http://www.codeguru.com/vb/openfaq/comments/159.shtml      *
'*   VB Programmer's SourceBook : Open FAQ                      *
'*   Submission by                                              *
'*            Pierre-Emmanuel Gros (1998/11/02)                 *
'*            pegros@mygale.org                                 *
'****************************************************************

Private Const WM_USER As Long = &H400
Private Const EM_FORMATRANGE As Long = WM_USER + 57
Private Const EM_SETTARGETDEVICE As Long = WM_USER + 72
Private Const PHYSICALOFFSETX As Long = 112
Private Const PHYSICALOFFSETY As Long = 113

Private Declare Function GetDeviceCaps Lib "gdi32" (ByVal hdc As Long, ByVal nIndex As Long) As Long

Private Declare Function SendMessage _
    Lib "user32" Alias "SendMessageA" _
    (ByVal hwnd As Long, ByVal Msg As Long, _
    ByVal wp As Long, lp As Any) _
As Long

Private Declare Function CreateDC _
    Lib "gdi32" Alias "CreateDCA" _
    (ByVal lpDriverName As String, ByVal lpDeviceName As String, _
    ByVal lpOutput As Long, ByVal lpInitData As Long) _
As Long

Private Type Rect
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private Type CharRange
    cpMin As Long ' First character of range (0 for start of doc)
    cpMax As Long ' Last character of range (-1 for end of doc)
End Type

Private Type FormatRange
    hdc As Long ' Actual DC to draw on
    hdcTarget As Long ' Target DC for determining text formatting
    rc As Rect ' Region of the DC to draw to (in twips)
    rcPage As Rect ' Region of the entire DC (page size) (intwips)
    chrg As CharRange ' Range of text to draw (see above declaration)
End Type




Public Sub PrintRTF(RTF As RichTextBox, LeftMarginWidth As Long, TopMarginHeight As Long, _
                    RightMarginWidth As Long, BottomMarginHeight As Long, Prn As Printer)

    Dim LeftOffset As Long, TopOffset As Long
    Dim LeftMargin As Long, TopMargin As Long
    Dim RightMargin As Long, BottomMargin As Long
    Dim fr As FormatRange
    Dim rcDrawTo As Rect
    Dim rcPage As Rect
    Dim TextLength As Long
    Dim NextCharPosition As Long
    Dim R As Long

    ' Start a print job to get a valid Prn.hDC
    Prn.Print Space(1)
    Prn.ScaleMode = vbTwips

    ' Get the offsett to the printable area on the page in twips
    LeftOffset = Prn.ScaleX(GetDeviceCaps(Prn.hdc, PHYSICALOFFSETX), vbPixels, vbTwips)
    TopOffset = Prn.ScaleY(GetDeviceCaps(Prn.hdc, PHYSICALOFFSETY), vbPixels, vbTwips)

    ' Calculate the Left, Top, Right, and Bottom margins
    LeftMargin = LeftMarginWidth - LeftOffset
    TopMargin = TopMarginHeight - TopOffset
    RightMargin = (Prn.Width - RightMarginWidth) - LeftOffset
    BottomMargin = (Prn.Height - BottomMarginHeight) - TopOffset

    ' Set printable area rect
    rcPage.Left = 0
    rcPage.Top = 0
    rcPage.Right = Prn.ScaleWidth
    rcPage.Bottom = Prn.ScaleHeight

    ' Set rect in which to print (relative to printable area)
    rcDrawTo.Left = LeftMargin
    rcDrawTo.Top = TopMargin
    rcDrawTo.Right = RightMargin
    rcDrawTo.Bottom = BottomMargin

    ' Set up the print instructions
    fr.hdc = Prn.hdc ' Use the same DC for measuring and rendering
    fr.hdcTarget = Prn.hdc ' Point at printer hDC
    fr.rc = rcDrawTo ' Indicate the area on page to drawto
    fr.rcPage = rcPage ' Indicate entire size of page
    fr.chrg.cpMin = 0 ' Indicate start of text through
    fr.chrg.cpMax = -1 ' end of the text

    ' Get length of text in RTF
    TextLength = Len(RTF.Text)

    ' Loop printing each page until done
    Do
        ' Print the page by sending EM_FORMATRANGE message
        NextCharPosition = SendMessage(RTF.hwnd, EM_FORMATRANGE, True, fr)
        If NextCharPosition >= TextLength Then Exit Do 'If done thenexit
        fr.chrg.cpMin = NextCharPosition ' Starting position for next Page
        Prn.NewPage ' Move on to next page
        Prn.Print Space(1) ' Re-initialize hDC
        fr.hdc = Prn.hdc
        fr.hdcTarget = Prn.hdc
    Loop

    'Commit the print job
    'Prn.EndDoc

    ' Allow the RTF to free up memory
    R = SendMessage(RTF.hwnd, EM_FORMATRANGE, False, ByVal CLng(0))

End Sub




